from . import res_config_settings
from . import auth_oauth_provider
from . import res_company
from . import res_partner
from . import subscription_request
from . import res_users
from . import account_invoice
from . import product
from . import utm_source
from . import crm_lead
from . import keycloak_sync_wiz
from . import cm_place
from . import res_users_role
